@@ -17,6 +17,7 @@ Relevant defines:
   CONFIGURE_CC - see emcc
 '''
 
+from __future__ import print_function
 import os, sys
 from tools import shared
 from subprocess import CalledProcessError
@@ -26,7 +27,7 @@ from subprocess import CalledProcessError
 #
 def run():
   if len(sys.argv) < 2 or ('configure' not in sys.argv[1] and 'cmake' not in sys.argv[1]):
-    print >> sys.stderr, '''
+    print('''
   emconfigure is a helper for configure, setting various environment
   variables so that emcc etc. are used. Typical usage:
 
@@ -34,16 +35,17 @@ def run():
 
   (but you can run any command instead of configure)
 
-  '''
+  ''', file=sys.stderr)
   elif 'cmake' in sys.argv[1]:
     node_js = shared.NODE_JS
-    if type(node_js) is list: node_js = ' '.join(node_js)
+    if type(node_js) is list: node_js = node_js[0]
+    node_js = shared.Building.which(node_js)
     node_js = node_js.replace('"', '\"')
     sys.argv = sys.argv[:2] + ['-DCMAKE_CROSSCOMPILING_EMULATOR="' + node_js +'"'] + sys.argv[2:]
 
   try:
     shared.Building.configure(sys.argv[1:])
-  except CalledProcessError, e:
+  except CalledProcessError as e:
     sys.exit(e.returncode)
 
 if __name__ == '__main__':
